/* 
 * BeOS DR8.2 version courtesy of Mike Perham (mperham@cs.cornell.edu)
 * I make no copyright or warranty clains on this code.
 * It is released AS IS.
 */

/*
 * Copyright (c) 1980, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/types.h>
#include <net/socket.h>
#include <net/netdb.h>
#include <stdio.h>
#include <getopt.h>

#define	NICHOST	"whois.internic.net"

void usage();

#ifdef DEBUG
#define msg(a) 	printf(a)
#else
#define msg(a)
#endif

int main(int argc, char **argv)
{
	extern char *optarg;
	extern int optind;
	struct sockaddr_in sin;
	struct hostent *hp;
	struct servent *sp;
	int s, amount;
	char ch;
	char *host;
	char ahost[MAXHOSTNAMELEN];

	host = NICHOST;
	while ((ch = getopt(argc, argv, "h:")) != EOF)
		switch((char)ch) {
            case 'h':
                host = optarg;
                break;
            case '?':
            default:
                usage();
		}
	argc -= optind;
	argv += optind;

	if (!argc)
		usage();

	hp = gethostbyname(host);
	if (hp == NULL) {
		fprintf(stderr, "whois: %s: ", host);
		herror((char *)NULL);
		exit(1);
	}
	host = hp->h_name;

	////////////// Open socket to whois server ///////////////////
	msg("Opening socket\n");
	s = socket(hp->h_addrtype, SOCK_STREAM, 0);
	if (s < 0) {
		perror("whois: socket");
		exit(1);
	}
	memset((caddr_t)&sin, 0, sizeof (sin));
	sin.sin_family = hp->h_addrtype;
	msg("Binding socket\n");
	if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
		perror("whois: bind");
		exit(1);
	}
	memcpy((char *)&sin.sin_addr, hp->h_addr, hp->h_length);
	/// BeOS does not define the whois port, so we need to
	/// hardcode it ....
	sin.sin_port = ntohs(43);
	msg("Connecting socket\n");
	if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
		perror("whois: connect");
		exit(1);
	}
	////////////////// End of socket setup /////////////////////
	while (argc-- > 1) {
	// It looks like we are sending multiple domains to the 
	// whois server ... This doesn't seem to work and the
	// Solaris version of whois does not support this so
	// I'm not going to worry about.  How many times have
	// you wanted to query multiple domains at once?
		msg(("Sending %s\n", *argv));
		sprintf(ahost, "%s ", *argv++);
		send(s, ahost, strlen(ahost), NULL);
	}
	msg(("Sending %s\n", *argv));
	sprintf(ahost,"%s\r\n", *argv);
	send(s, ahost, strlen(ahost), NULL);
	msg("Receiving data\n");
	amount = 1;
	while (amount > 0) {
		// Read data from whois server
        amount = recv(s, ahost, sizeof(ahost), NULL);
        if (amount > 0) {
        	// We have to null terminate the string
            ahost[amount] = '\0';
            printf(ahost);
        }
    }
	msg("Closing socket\n");
	closesocket(s);
	return(0);
}

void usage()
{
	fprintf(stderr, "usage: whois [-h hostname] domain\n");
	exit(1);
}
